/*
 * Compress.cpp
 * 
 * Written by Matt Pauker
 *
 * 5/11/96
 */

#include "Compress.h"
#include "StatusWindow.h"

extern char **environ;

void DoError(const char *errorMsg)
// Displays the string in errorMsg in a BAlert
{
	BAlert *myAlert;
	
	myAlert = new BAlert("", errorMsg, "Cancel", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
	myAlert->Go();
}

void CopyExtraFields(BRecord *sourceRecord, BRecord **destRecord)
// Copies all "extra" fields from sourceRecord to destRecord
{
	long x, size;
	char *fieldName;
	void *data;

	x = 0;
	
	while (sourceRecord->GetExtraInfo(x, &fieldName, &size) == B_NO_ERROR)
	{
		data = sourceRecord->FindExtra(fieldName, &size);
		(*destRecord)->SetExtra(fieldName, data, size);
		x++;
	}
}
	
void CreateTempFile(BFile *tempFile, BDirectory *tempDir)
// Creates a temporary file in the /tmp directory and returns its record_ref
{
	record_ref dirRef;
	
	get_ref_for_path("/tmp", &dirRef);
	tempDir->SetRef(dirRef);
	tempDir->Create("CompressTemp", tempFile, NULL, NULL, NULL);
}

#pragma export on
void process_refs(record_ref dirRef, BMessage *message, void *data);
#pragma export reset

void process_refs(record_ref dirRef, BMessage *message, void *data)
{
	ulong type;
	long numRefs;
	long result;
	
	message->GetInfo("refs", &type, &numRefs);
	
	if (numRefs == 0) /* Nothing selected */
		DoError("You must select a file to compress.");
	else
	{
		BDirectory dir;

		dir.SetRef(dirRef);
		
		if (dir.Error() != B_NO_ERROR)
			DoError("You must select a file within a folder.");
		else
		{
			record_ref ref;
			char *path = new char[1000];
			long x;

			// Set the current directory to dir, so that gzip can find the file
			dir.GetPath(path, 1000);
			chdir(path);
	
			for (x = 0; x < numRefs; x++)
			{
				ref = message->FindRef("refs", x);
				if (does_ref_conform(ref, "File"))
				{
					BFile file, tempFile;
					BDirectory tempDir;
					char *fileName = new char[B_FILE_NAME_LENGTH];
					ulong type, app;
					BRecord *record;
					BFile newFile;
					
					file.SetRef(ref);
					file.GetName(fileName);
					
					// Store the type and app so they can be stored in extra fields and retrieved
					// at decompression time
					file.GetTypeAndApp(&type, &app);

					// The file's extra fields need to be stored, so create a temp file in which to
					// store them.
					CreateTempFile(&tempFile, &tempDir);
					record = tempFile.Record();
					CopyExtraFields(file.Record(), &record);
										
					char **passargv;
					thread_id gzipThread;
					thread_info info;

					// Set up the arguments to send to gzip
					passargv = (char **)malloc((3) * (sizeof(char *)));
				
					passargv[0] = "/boot/bin/gzip";
					passargv[1] = fileName;
					passargv[2] = 0;
			
					// Try to load gzip
					gzipThread = load_posix(passargv[0], passargv, environ);
					if (gzipThread > 0)
					{
						resume_thread(gzipThread);	// Start it
						
						snooze(400000.0);		// Wait a bit
						
						// If it's still running (File must have been large), show a status
						// window
						if (get_thread_info(gzipThread, &info) == B_NO_ERROR)
						{
							StatusWindow *statWindow;

							statWindow = new StatusWindow();
							statWindow->Show();
							wait_for_thread(gzipThread, &result);
							statWindow->Close();
						}

						// Get the new file
						strcat(fileName, ".gz");
						dir.GetFile(fileName, &newFile);
						record = newFile.Record();
						
						// Add the type and app extra fields
						record->SetExtra("compressType", (const void *)&type, sizeof(ulong));
						record->SetExtra("compressApp", (const void *)&app, sizeof(ulong));

						// Add the extra fields
						CopyExtraFields(tempFile.Record(), &record);
						record->Commit();
					}
					else
						DoError("Sorry, I couldn't find the gzip tool.");
					
					free(passargv);
					delete[] fileName;
					tempDir.Remove(&tempFile);	
					
				}
				else
					DoError("You can only compress files.");
			}
		}
	}
}